@@ -9,31 +9,42 @@ from numpy.testing import assert_almost_equal
 class TestCommandLineRunner:
     """Testing the command line interface."""
 
-    # @pytest.mark.skip("Will be an infinity loop")
-    # def test_version(self, monkeypatch, script_runner):
-    #     """Testing the version command."""
-    #     import pytest
-    #     from spectrafit import __version__
-    #     monkeypatch.setattr("builtins.input", lambda _: "y")
-    #     ret = script_runner.run(
-    #         "spectrafit",
-    #         "spectrafit/test/test_data.txt",
-    #         "-i",
-    #         "spectrafit/test/test_input_1.json",
-    #     )
-    #
-    #     assert ret.success
-    #     assert ret.stdout == f"Currently used version is: {__version__}\n"
-    #     assert ret.stderr == ""
+    def test_version(self, monkeypatch, script_runner):
+        """Testing the version command."""
+        from spectrafit import __version__
+
+        monkeypatch.setattr("builtins.input", lambda _: "y")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_input_1.json",
+        )
+
+        assert ret.success
+        assert ret.stdout == f"Currently used version is: {__version__}\n"
+        assert ret.stderr == ""
 
     def test_extended(self, monkeypatch, script_runner):
         """Testing the extended command."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.txt",
+            "_",
             "-i",
-            "spectrafit/test/test_input_2.json",
+            "spectrafit/test/scripts/test_input_2.json",
+        )
+        assert ret.success
+        assert ret.stderr == ""
+
+    def test_extended_verbose(self, monkeypatch, script_runner):
+        """Testing the extended with verbose command."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_input_7.json",
         )
         assert ret.success
         assert ret.stderr == ""
@@ -47,64 +58,70 @@ class TestFileFormat:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_3.json",
+            "spectrafit/test/scripts/test_input_3.json",
             "-o",
-            "result_json",
+            "spectrafit/test/export/result_json",
         )
         assert ret.success
         assert ret.stderr == ""
-        assert len(list(Path(".").glob("result_json*.json"))) == 1
-        assert len(list(Path(".").glob("result_json*.csv"))) == 3
+        assert (
+            len(list(Path(".").glob("spectrafit/test/export/result_json*.json"))) == 1
+        )
+        assert len(list(Path(".").glob("spectrafit/test/export/result_json*.csv"))) == 3
 
     def test_yml_input(self, monkeypatch, script_runner):
         """Testing yml support."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_3.yml",
+            "spectrafit/test/scripts/test_input_3.yml",
             "-o",
-            "result_yml",
+            "spectrafit/test/export/result_yml",
         )
         assert ret.success
         assert ret.stderr == ""
-        assert len(list(Path(".").glob("result_yml*.json"))) == 1
-        assert len(list(Path(".").glob("result_yml*.csv"))) == 3
+        assert len(list(Path(".").glob("spectrafit/test/export/result_yml*.json"))) == 1
+        assert len(list(Path(".").glob("spectrafit/test/export/result_yml*.csv"))) == 3
 
     def test_yaml_input(self, monkeypatch, script_runner):
         """Testing yaml support."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_3.yaml",
+            "spectrafit/test/scripts/test_input_3.yaml",
             "-o",
-            "result_yaml",
+            "spectrafit/test/export/result_yaml",
         )
         assert ret.success
         assert ret.stderr == ""
-        assert len(list(Path(".").glob("result_yaml*.json"))) == 1
-        assert len(list(Path(".").glob("result_yaml*.csv"))) == 3
+        assert (
+            len(list(Path(".").glob("spectrafit/test/export/result_yaml*.json"))) == 1
+        )
+        assert len(list(Path(".").glob("spectrafit/test/export/result_yaml*.csv"))) == 3
 
     def test_toml_input(self, monkeypatch, script_runner):
         """Testing toml support."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_3.toml",
+            "spectrafit/test/scripts/test_input_3.toml",
             "-o",
-            "result_toml",
+            "spectrafit/test/export/result_toml",
         )
         assert ret.success
         assert ret.stderr == ""
-        assert len(list(Path(".").glob("result_toml*.json"))) == 1
-        assert len(list(Path(".").glob("result_toml*.csv"))) == 3
+        assert (
+            len(list(Path(".").glob("spectrafit/test/export/result_toml*.json"))) == 1
+        )
+        assert len(list(Path(".").glob("spectrafit/test/export/result_toml*.csv"))) == 3
 
 
 class TestFileFormatOutput:
@@ -115,12 +132,14 @@ class TestFileFormatOutput:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         _ = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.txt",
+            "_",
             "-i",
-            "spectrafit/test/test_input_2.json",
+            "spectrafit/test/scripts/test_input_2.json",
         )
-        assert len(list(Path(".").glob("spectrafit/test/fit_results*.json"))) == 1
-        assert len(list(Path(".").glob("spectrafit/test/fit_results*.csv"))) == 3
+        assert (
+            len(list(Path(".").glob("spectrafit/test/export/fit_results*.json"))) == 1
+        )
+        assert len(list(Path(".").glob("spectrafit/test/export/fit_results*.csv"))) == 3
 
 
 class TestMoreFeatures:
@@ -131,9 +150,9 @@ class TestMoreFeatures:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_4.json",
+            "spectrafit/test/scripts/test_input_4.json",
         )
         assert ret.success
         assert ret.stderr == ""
@@ -143,18 +162,18 @@ class TestMoreFeatures:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_5.json",
+            "spectrafit/test/scripts/test_input_5.json",
             "-o",
-            "spectrafit/test/e0_result",
+            "spectrafit/test/export/e0_result",
             "-e0",
             "0.0",
         )
         assert ret.success
         assert ret.stderr == ""
 
-        df_test = pd.read_csv(Path("./spectrafit/test/e0_result_fit.csv"))
+        df_test = pd.read_csv(Path("./spectrafit/test/export/e0_result_fit.csv"))
         assert_almost_equal(df_test["energy"].min(), 0.0, decimal=0)
 
     def test_energyrange_e1(self, monkeypatch, script_runner):
@@ -162,11 +181,11 @@ class TestMoreFeatures:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_5.json",
+            "spectrafit/test/scripts/test_input_5.json",
             "-o",
-            "spectrafit/test/e1_result",
+            "spectrafit/test/export/e1_result",
             "--oversampling",
             "-e1",
             "5.0",
@@ -174,7 +193,7 @@ class TestMoreFeatures:
         assert ret.success
         assert ret.stderr == ""
 
-        df_test = pd.read_csv(Path("./spectrafit/test/e1_result_fit.csv"))
+        df_test = pd.read_csv(Path("./spectrafit/test/export/e1_result_fit.csv"))
         assert_almost_equal(df_test["energy"].max(), 5.0, decimal=0)
 
     def test_energyrange_e0e1(self, monkeypatch, script_runner):
@@ -182,11 +201,11 @@ class TestMoreFeatures:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_5.json",
+            "spectrafit/test/scripts/test_input_5.json",
             "-o",
-            "spectrafit/test/e0e1_result",
+            "spectrafit/test/export/e0e1_result",
             "--oversampling",
             "-e0",
             "0",
@@ -196,7 +215,7 @@ class TestMoreFeatures:
         assert ret.success
         assert ret.stderr == ""
 
-        df_test = pd.read_csv(Path("./spectrafit/test/e0e1_result_fit.csv"))
+        df_test = pd.read_csv(Path("./spectrafit/test/export/e0e1_result_fit.csv"))
         assert_almost_equal(df_test["energy"].max(), 5.0, decimal=0)
         assert_almost_equal(df_test["energy"].min(), 0.0, decimal=0)
 
@@ -205,20 +224,20 @@ class TestMoreFeatures:
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "_",
             "-i",
-            "spectrafit/test/test_input_all_models.toml",
+            "spectrafit/test/scripts/test_input_all_models.toml",
         )
         assert ret.success
         assert ret.stderr == ""
 
-    def test_not_allowed_input(self, monkeypatch, script_runner):
+    def test_not_allowed_input_1(self, monkeypatch, script_runner):
         """Testing test all models of spectrafit."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
-        fname = "spectrafit/test/test_wrong.pp"
+        fname = "spectrafit/test/scripts/test_wrong.pp"
         ret = script_runner.run(
             "spectrafit",
-            "spectrafit/test/test_data.csv",
+            "spectrafit/test/import/test_data.csv",
             "-i",
             fname,
         )
@@ -228,8 +247,32 @@ class TestMoreFeatures:
             "Supported fileformats are: '*.json', '*.yaml', and '*.toml'\n"
         )
 
+    def test_not_allowed_input_2(self, monkeypatch, script_runner):
+        """Testing missing mininizmer parameter in input."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_missing_parameters_1.json",
+        )
+        assert not ret.success
+        assert ret.stderr == "Missing 'minimizer' in 'parameters'!\n"
+
+    def test_not_allowed_input_3(self, monkeypatch, script_runner):
+        """Testing missing optimizer parameter in input."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_missing_parameters_2.json",
+        )
+        assert not ret.success
+        assert ret.stderr == "Missing key 'optimizer' in 'parameters'!\n"
+
     def test_no_input(self, monkeypatch, script_runner):
-        """Testing test all models of spectrafit."""
+        """Testing no provided input for spectrafit."""
         monkeypatch.setattr("builtins.input", lambda _: "n")
         ret = script_runner.run(
             "spectrafit",
@@ -238,3 +281,107 @@ class TestMoreFeatures:
             "spectrafit/test/no_input.pp",
         )
         assert not ret.success
+
+    def test_conf_interval(self, monkeypatch, script_runner):
+        """Testing upper energy range cut."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_input_6.json",
+            "-o",
+            "spectrafit/test/export/conf_interval_result",
+        )
+        assert ret.success
+        assert ret.stderr == ""
+        assert (
+            len(
+                list(
+                    Path(".").glob("spectrafit/test/export/conf_interval_result*.json")
+                )
+            )
+            == 1
+        )
+
+    def test_get_no_errors(self, monkeypatch, script_runner):
+        """Testing for no errorbars for spectrafit."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_input_8.json",
+        )
+        assert ret.success
+
+    def test_load_noglobal(self, monkeypatch, script_runner):
+        """Testing for no errorbars for spectrafit."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit",
+            "_",
+            "-i",
+            "spectrafit/test/scripts/test_input_8.json",
+            "-g",
+            "0",
+        )
+        assert ret.success
+
+    def test_non_numeric_data(self, monkeypatch, script_runner):
+        """Testing missing mininizmer parameter in input."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit", "_", "-i", "spectrafit/test/scripts/test_input_9.json"
+        )
+        assert not ret.success
+
+
+class TestGlobalFitting:
+    """Test class for global fitting."""
+
+    def test_global_fitting_0(self, monkeypatch, script_runner):
+        """Testing global fitting."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit", "_", "-i", "spectrafit/test/scripts/test_input_global_0.json"
+        )
+        assert ret.success
+        assert ret.stderr == ""
+        assert (
+            len(
+                list(
+                    Path(".").glob("spectrafit/test/export/global_fit_results_0*.json")
+                )
+            )
+            == 1
+        )
+        assert (
+            len(
+                list(Path(".").glob("spectrafit/test/export/global_fit_results_0*.csv"))
+            )
+            == 3
+        )
+
+    def test_global_fitting_1(self, monkeypatch, script_runner):
+        """Testing global fitting."""
+        monkeypatch.setattr("builtins.input", lambda _: "n")
+        ret = script_runner.run(
+            "spectrafit", "_", "-i", "spectrafit/test/scripts/test_input_global_1.json"
+        )
+        assert ret.success
+        assert ret.stderr == ""
+        assert (
+            len(
+                list(
+                    Path(".").glob("spectrafit/test/export/global_fit_results_1*.json")
+                )
+            )
+            == 1
+        )
+        assert (
+            len(
+                list(Path(".").glob("spectrafit/test/export/global_fit_results_1*.csv"))
+            )
+            == 3
+        )
